resource_name :container
property :container_name, String, name_property: true
property :options, Hash, default: {}

def lxc_container(name)
  require 'lxc'
  return LXC::Container.new(name)
end

action :start do
  ct = lxc_container(new_resource.container_name)
  return if ct.running?
  converge_by("start container '#{ct.name}'") do
    ct.start
  end
end

action :stop do
  ct = lxc_container(new_resource.container_name)
  return unless ct.running?
  converge_by("stop container '#{ct.name}'") do
    ct.stop
  end
end

action :create do
  ct = lxc_container(new_resource.container_name)
  return if ct.defined?
  converge_by("create container '#{ct.name}'") do
    opts = @new_resource.options
    template = opts[:template] || 'ubuntu'
    block_device = opts[:block_device] || nil
    template_options = opts[:template_options] || []
    flags = opts[:flags] || 0
    ct.create(template, block_device, flags, template_options)
  end
end

action :destroy do
  ct = lxc_container(new_resource.container_name)
  return unless ct.defined?
  converge_by("destroy container '#{ct.name}'") do
    ct.destroy
  end
end
