resource_name :container_user
property :username, String, name_attribute: true
property :user_password, String
property :create_user, [TrueClass, FalseClass], default: true
property :home_dir, String, required: true
property :container_dir, String
property :veth_limit, Integer, default: 100
default_action :setup

def subuid_info(container_user)
  ::File.read('/etc/subuid').scan(/#{container_user}:(\d+):(\d+)/).flatten
end

def subgid_info(container_user)
  ::File.read('/etc/subgid').scan(/#{container_user}:(\d+):(\d+)/).flatten
end

action :setup do
  home_dir = new_resource.home_dir
  container_user = new_resource.username
  limit = new_resource.veth_limit
  usernet_file = '/etc/lxc/lxc-usernet'
  line = "#{container_user} veth lxcbr0 #{limit}"
  if new_resource.container_dir
    container_dir = new_resource.container_dir
  else
    container_dir = "#{home_dir}/.local/share/lxc"
  end

  user container_user do
    home home_dir
    shell '/bin/bash'
    password new_resource.user_password if new_resource.user_password
    supports(manage_home: true)
    only_if { new_resource.create_user }
  end

  subuid_start, subuid_range = subuid_info(container_user)
  subgid_start, subgid_range = subgid_info(container_user)

  %w(
    .config
    .local
    .local/share
    .cache
    lxc.conf.d
    .config/lxc
    .local/share/lxc
    .local/share/lxcsnaps
    .cache/lxc
  ).each do |dir|
    directory "#{home_dir}/#{dir}" do
      user container_user
      group container_user
      mode 0775
    end
  end

  template "#{home_dir}/.config/lxc/default.conf" do
    owner container_user
    group container_user
    mode 0644
    source 'lxc.conf.erb'
    cookbook 'container'
    variables(
      subuid_start: subuid_start,
      subuid_range: subuid_range,
      subgid_start: subgid_start,
      subgid_range: subgid_range
    )
  end
  file "#{home_dir}/.config/lxc/lxc.conf" do
    owner container_user
    group container_user
    mode 0644
    content "lxc.lxcpath = #{container_dir}"
  end

  ruby_block 'update lxc-usernet' do
    block do
      ::File.open(usernet_file, 'a+') do |f|
        f.write("#{line}\n")
      end
    end
    not_if do
      ::File.exist?(usernet_file) &&
        ::File.read(usernet_file).lines.any? { |l| l.chomp == line }
    end
  end

  file '/etc/lxc/lxc-usernet' do
    owner 'root'
    group 'root'
    mode 0644
  end
end
